from sqlobject import *
from datetime import datetime
from turbogears import identity 
from turbogears.database import PackageHub

hub = PackageHub("webhattg")
__connection__ = hub

class Ticket(SQLObject):
	name = UnicodeCol()
	reporter = UnicodeCol()
	body = UnicodeCol()
	category = ForeignKey("Category")
	owner = ForeignKey("Person")
	priority = EnumCol(enumValues=['normal','low','high'],default="normal")
	created = DateTimeCol(default=datetime.now)
	closed = BoolCol(default=False)
	status = EnumCol(enumValues=['new','assigned','closed'],default="new")
	resolution = ForeignKey("Resolution")
	comments = MultipleJoin("Comment")

class Announcement(SQLObject):
    name = StringCol()
    date = DateCol()
    description = UnicodeCol()
    published = BoolCol(default=False)
    category = StringCol()
    tickets = RelatedJoin("Ticket", intermediateTable="announcement_ticket",
                          joinColumn="ticket_id", otherColumn="announcement_id")

class Category(SQLObject):
    name = StringCol()
    tickets = MultipleJoin("Ticket")

class Resolution(SQLObject):
    name = StringCol()
    tickets = MultipleJoin("Ticket")

class Comment(SQLObject):
    title = UnicodeCol()
    created = DateTimeCol(default=datetime.now)
    author = UnicodeCol()
    body = UnicodeCol()
    ticket = ForeignKey("Ticket")

class Person(SQLObject):
    displayName     = StringCol(length=255, alternateID=True)
    user_name       = StringCol(length=16, alternateID=True,
                                alternateMethodName="by_user_name")
    password        = StringCol()
    emailAddress    = StringCol(length=255, alternateID=True,
                                alternateMethodName="by_email_address")
    
    groups = RelatedJoin("Groups", intermediateTable="tg_user_group",
                         joinColumn="user_name", otherColumn="group_name")
    
    def _get_permissions( self ):
        perms = set()
        for g in self.groups:
            perms = perms | set(g.permissions)
        return perms

class StudentAccount(SQLObject):
	name = StringCol(length=16, alternateID=True,
				alternateMethodName="by_user_name")
	password = StringCol()

class Groups(SQLObject):
    group_name  = StringCol(length=16, alternateID=True,
                            alternateMethodName="by_group_name")
    displayName = StringCol(length=255)
    created     = DateTimeCol(default=datetime.now)
    
    # collection of all users belonging to this group
    users       = RelatedJoin("Person", 
                              intermediateTable="tg_user_group",
                              joinColumn="group_name", 
                              otherColumn="user_name")
    
    # collection of all permissions for this group
    permissions = RelatedJoin("Permission", 
                              joinColumn="group_name", 
                              intermediateTable="tg_group_permission",
                              otherColumn="permission_name")

class Permission(SQLObject):
    permission_name = StringCol(length=16, alternateID=True,
                                alternateMethodName="by_permission_name")
    description     = StringCol(length=255)
    groups          = RelatedJoin("Groups", 
                                  intermediateTable="tg_group_permission",
                                  joinColumn="permission_name", 
                                  otherColumn="group_name")

class VisitIP(SQLObject):
    
    class sqlmeta:
        table = 'visit_ip'
        
    visit = ForeignKey('TG_Visit')
    ip_address = StringCol(length=20)

class ResourceCount(SQLObject):
    resource_name = StringCol(length=40, alternateID=True,
                              alternateMethodName="by_resource_name")
    hit_count = IntCol(default=1)
    first_access = DateTimeCol(default=datetime.now)
    last_access = DateTimeCol()

class User(SQLObject):
    user_name = UnicodeCol()
    user_email = StringCol(length=255, alternateID=True,
                           alternateMethodName="by_author_email")
    evaluations = MultipleJoin("Evaluation")

class Purpose(SQLObject):
    name = StringCol()
    date = DateTimeCol()
    expired = BoolCol(default=False)
    evaluations = MultipleJoin("Evaluation")

class Evaluation(SQLObject):
    author = ForeignKey("User")
    purpose = ForeignKey("Purpose")
    created = DateTimeCol(default=datetime.now)
    closed = BoolCol(default=False)
    record = SingleJoin("Record")

class Record(SQLObject):
    evaluation = ForeignKey("Evaluation")
    created = DateTimeCol(default=datetime.now)
    updated = DateTimeCol()
    content = StringCol()


