import logging
import cherrypy

import turbogears
from turbogears import controllers, expose, validate, redirect, identity
from turbogears import exception_handler

from webhattg import json, utils
from webhattg.model import StudentAccount
log = logging.getLogger("webhattg.subcontrollers.StudentAccountController")

class StudentAccountController(identity.SecureResource):

	@expose(template="webhattg.templates.StudentAccountAssistantPage")
	def index(self):
		pass
		return dict(active_section = 'newstudentaccount')

	@expose(template="webhattg.templates.WebHatSystemInternalErrorPage")
	def webhat_system_internal_error_handler(self,tg_exceptions=None):
		dictErrorMsg = tg_exceptions.args[0]
		return dict(active_section = 'newstudentaccount',
				handling_index=True, 
				exception=str(tg_exceptions), 
				message=dictErrorMsg['message'])

	@expose(template="webhattg.templates.SuccessNewAccountPage")
	@exception_handler(webhat_system_internal_error_handler, rules="isinstance(tg_exceptions,ValueError)")
	def apply_for_new_account(self, account_name, password):
	
		def check_account_existance(studentAccount):
			SVN_REPOSITORY_ROOT = "C:\\svn-win32-1.4.0\\svn-repos"
			nameToCheck = studentAccount.name + "-repos"
			import os
			allSvnRepos = os.listdir(SVN_REPOSITORY_ROOT)
			if nameToCheck in allSvnRepos:
				return True
			return False
		
		def account_validation(studentAccount):
			
			accountName = studentAccount.name
			# check if the account length is right
			if len(accountName) != 9:
				return False
			# check if the first english digit is alpha and upper case
			firstAlpha = accountName[0]
			if firstAlpha.isalpha() == False or (firstAlpha.upper() != firstAlpha):
				return False
			# check if the rest part are all numbers
			for restNumber in accountName[1:-1]:
				if restNumber.isdigit == False:
					return False
			return True
		
		def create_environment(studentAccount):
			utils.writeLog('[AccountCreation] '+studentAccount.name+','+studentAccount.password)
			# create subversion repository
			SVN_CREATED_REPOSITORY_PATH = self.create_subversion_repository(studentAccount)
			# set trac environment
			self.setTracEnvironment(studentAccount, SVN_CREATED_REPOSITORY_PATH)
			utils.writeLog('[AccountCreated] '+studentAccount.name+','+studentAccount.password)
			# start server
			self.start_svn_and_trac_serve()
		
		studentAccount = StudentAccount(name=account_name, password=password)
		
		if studentAccount.name == '' or studentAccount.password == '':
			dictErrorMsg = {}
			dictErrorMsg['message'] = "The account name or password can not be empty."
			raise ValueError(dictErrorMsg)
			
		if check_account_existance(studentAccount):
			dictErrorMsg = {}
			dictErrorMsg['message'] = "The account name is already existed."
			raise ValueError(dictErrorMsg)
		
		if account_validation(studentAccount) == False:
			dictErrorMsg = {}
			dictErrorMsg['message'] = "The account name fromat is not correct."
			raise ValueError(dictErrorMsg)
		
		'''
		# create subversion repository
		SVN_CREATED_REPOSITORY_PATH = self.create_subversion_repository(studentAccount)
		# set trac environment
		self.setTracEnvironment(studentAccount, SVN_CREATED_REPOSITORY_PATH)
		# start server
		self.start_svn_and_trac_serve()
		'''
		# threading
		import thread
		thread.start_new(create_environment, (studentAccount,))
		
		idNum = int(studentAccount.name[7])
		port = 8000
		if idNum % 10 == 0 or idNum % 10 == 1:
			port = port + 0	
		elif idNum % 10 == 2 or idNum % 10 == 3:
			port = port + 2	
		elif idNum % 10 == 4 or idNum % 10 == 5:
			port = port + 4	
		elif idNum % 10 == 6 or idNum % 10 == 7:
			port = port + 6	
		elif idNum % 10 == 8 or idNum % 10 == 9:
			port = port + 8		
		
		return dict(active_section = 'newstudentaccount',
				succ_name=studentAccount.name, 
				succ_password=studentAccount.password,
				port=port)
	
	
	def create_subversion_repository(self, studentAccount):
		
		SVN_REPOSITORY_ROOT = "C:\\svn-win32-1.4.0\\svn-repos"
		REPOSITORY_TO_CREATE = studentAccount.name + "-repos"
		SVN_TEMPLATES_ROOT = "C:\\svn-win32-1.4.0\\templates"
		SVN_REPOSITORY_ROOT_FOR_IMPORT = "C:/svn-win32-1.4.0/svn-repos"
		SVN_TEMPLATES_ROOT_FOR_IMPORT = "C:/svn-win32-1.4.0/templates"
		
		SVN_CREATED_REPOSITORY_PATH = SVN_REPOSITORY_ROOT + '\\' + REPOSITORY_TO_CREATE 
		SVN_CREATED_REPOSITORY_PATH_FOR_IMPORT = SVN_REPOSITORY_ROOT_FOR_IMPORT + '/' + REPOSITORY_TO_CREATE 
		
		import os
		cmd = 'svnadmin create \"' + SVN_CREATED_REPOSITORY_PATH +'\"'
		os.system(cmd)
		
		cmd = 'copy ' + SVN_TEMPLATES_ROOT + '\\svnserve.conf ' + SVN_CREATED_REPOSITORY_PATH + '\\conf\\svnserve.conf'
		os.system(cmd)
		
		passwdFile = open(SVN_CREATED_REPOSITORY_PATH + '\\conf\\passwd', 'a')
		name = studentAccount.name
		password = studentAccount.password
		passwdFile.writelines(name+' = '+password)
		passwdFile.close()
		
		cmd = 'svn import ' + SVN_TEMPLATES_ROOT_FOR_IMPORT + '/dir-template ' + 'file:///' + SVN_CREATED_REPOSITORY_PATH_FOR_IMPORT + ' -m \"first import init dirs\"'
		os.system(cmd)
		
		return SVN_CREATED_REPOSITORY_PATH
		
	def setTracEnvironment(self, studentAccount, SVN_CREATED_REPOSITORY_PATH):
		
		def generate_auth_string(accountName, password):
			import md5
			realm = 'trac'
			kd = lambda x: md5.md5(':'.join(x)).hexdigest()
			authString = ':'.join((accountName, realm, kd([accountName, realm, password])))
			return authString
		
		TRAC_ENV_ROOT = "C:/TracEnv"
		TRAC_PROJ_NAME = studentAccount.name + "-trac"
		TRAC_TEMPLATE_ROOT = "C:/Python24/share/trac/templates"
		
		TRAC_PROJ_PATH = TRAC_ENV_ROOT + '/' + TRAC_PROJ_NAME
		
		import os
		cmd = 'trac-admin.py ' + TRAC_PROJ_PATH + ' initenv ' + TRAC_PROJ_NAME + ' sqlite:db/trac.db svn ' + SVN_CREATED_REPOSITORY_PATH  + ' ' + TRAC_TEMPLATE_ROOT
		os.system(cmd)
		
		authString = generate_auth_string(studentAccount.name, studentAccount.password)
		fileHtdigest = open(TRAC_PROJ_PATH + '/conf/htdigest','w')
		fileHtdigest.writelines(authString)
		fileHtdigest.close()
		
		cmd = 'trac-admin.py ' + TRAC_PROJ_PATH + ' permission remove anonymous *'
		print cmd
		os.system(cmd)
		
		cmd = 'trac-admin.py ' + TRAC_PROJ_PATH + ' permission add ' + studentAccount.name + ' TRAC_ADMIN'
		print cmd
		os.system(cmd)
		
		cmd = 'trac-admin.py ' + TRAC_PROJ_PATH + ' permission list'
		os.system(cmd)
	
	
	# [seLain:] this new version of start_svn_and_trac_serve , for IntroCS purpose, will not cause problem in windows
	# due to the command line length limitation
	def start_svn_and_trac_serve(self):
		
		TRACD_POSITION = "C:/Python24/Scripts/tracd"
		TRAC_ENV_ROOT = "C:/TracEnv"
		SVN_REPOSITORY_ROOT = "C:\\svn-win32-1.4.0\\svn-repos"
		
		# randomly pause re-start of tracd and subversion
		import time, random
		waitSecs = random.randint(180, 360)
		print 'waitSecs:', waitSecs
		time.sleep(waitSecs)
		
		## tracd start 
		# get all trac projects
		import os
		allTracProj = os.listdir(TRAC_ENV_ROOT)
		
		# get configuration strings
		authConfString = ['','','','','']
		projConfString = ['','','','','']
		for tracProj in allTracProj :
			idNum = int(tracProj[7])
			if idNum % 10 == 0 or idNum % 10 == 1:
				authConfString[0] = authConfString[0] + '--auth ' + tracProj + ',' + TRAC_ENV_ROOT + '/' + tracProj + '/conf/htdigest,trac '
				projConfString[0] = projConfString[0] + TRAC_ENV_ROOT + '/' + tracProj + ' '
			elif idNum % 10 == 2 or idNum % 10 == 3:
				authConfString[1] = authConfString[1] + '--auth ' + tracProj + ',' + TRAC_ENV_ROOT + '/' + tracProj + '/conf/htdigest,trac '
				projConfString[1] = projConfString[1] + TRAC_ENV_ROOT + '/' + tracProj + ' '
			elif idNum % 10 == 4 or idNum % 10 == 5:
				authConfString[2] = authConfString[2] + '--auth ' + tracProj + ',' + TRAC_ENV_ROOT + '/' + tracProj + '/conf/htdigest,trac '
				projConfString[2] = projConfString[2] + TRAC_ENV_ROOT + '/' + tracProj + ' '
			elif idNum % 10 == 6 or idNum % 10 == 7:
				authConfString[3] = authConfString[3] + '--auth ' + tracProj + ',' + TRAC_ENV_ROOT + '/' + tracProj + '/conf/htdigest,trac '
				projConfString[3] = projConfString[3] + TRAC_ENV_ROOT + '/' + tracProj + ' '
			elif idNum % 10 == 8 or idNum % 10 == 9:
				authConfString[4] = authConfString[4] + '--auth ' + tracProj + ',' + TRAC_ENV_ROOT + '/' + tracProj + '/conf/htdigest,trac '
				projConfString[4] = projConfString[4] + TRAC_ENV_ROOT + '/' + tracProj + ' '
		
		# stop tracd first
		cmd = 'sc stop tracd'
		os.system(cmd)
		# then restart tracd, distributed into five ports
		cmd = 'start /min ' + 'python ' + TRACD_POSITION + ' --port 8000 ' + authConfString[0] + projConfString[0]
		os.system(cmd)
		cmd = 'start /min ' + 'python ' + TRACD_POSITION + ' --port 8002 ' + authConfString[1] + projConfString[1]
		os.system(cmd)
		cmd = 'start /min ' + 'python ' + TRACD_POSITION + ' --port 8004 ' + authConfString[2] + projConfString[2]
		os.system(cmd)
		cmd = 'start /min ' + 'python ' + TRACD_POSITION + ' --port 8006 ' + authConfString[3] + projConfString[3]
		os.system(cmd)
		cmd = 'start /min ' + 'python ' + TRACD_POSITION + ' --port 8008 ' + authConfString[4] + projConfString[4]
		os.system(cmd)
	
		## svnserve start 
		# stop svnserve first
		cmd = 'sc stop svnserve'
		os.system(cmd)
		# then restart svnserve
		cmd = 'start /min svnserve --daemon -r ' + SVN_REPOSITORY_ROOT
		print cmd
		os.system(cmd)
		print 'end of svnserve start /min'
	
	'''
	# [seLain:] this previous version of start_svn_and_trac_serve , for general purpose, will cause problem in windows
	# due to the command line length limitation
	def start_svn_and_trac_serve(self):
		
		TRACD_POSITION = "C:/Python24/Scripts/tracd"
		TRAC_ENV_ROOT = "C:/TracEnv"
		SVN_REPOSITORY_ROOT = "C:\\svn-win32-1.4.0\\svn-repos"
		
		# randomly pause re-start of tracd and subversion
		import time, random
		waitSecs = random.randint(60, 180)
		print 'waitSecs:', waitSecs
		time.sleep(waitSecs)
		
		## tracd start 
		# get all trac projects
		import os
		allTracProj = os.listdir(TRAC_ENV_ROOT)
		
		# get configuration strings
		authConfString = ''
		projConfString = ''
		for tracProj in allTracProj :
			authConfString = authConfString + '--auth ' + tracProj + ',' + TRAC_ENV_ROOT + '/' + tracProj + '/conf/htdigest,trac '
			projConfString = projConfString + TRAC_ENV_ROOT + '/' + tracProj + ' '
		
		# stop tracd first
		cmd = 'sc stop tracd'
		os.system(cmd)
		# then restart tracd
		cmd = 'start /min ' + 'python ' + TRACD_POSITION + ' --port 8000 ' + authConfString + projConfString
		print cmd
		os.system(cmd)
		print 'end of tracd' 
		
		## svnserve start 
		# stop svnserve first
		cmd = 'sc stop svnserve'
		os.system(cmd)
		# then restart svnserve
		cmd = 'start /min svnserve --daemon -r ' + SVN_REPOSITORY_ROOT
		print cmd
		os.system(cmd)
		print 'end of svnserve start /min'
	'''