# subcontroller/announcement.py

import turbogears as tg
import cherrypy as http
from turbogears import expose, identity, redirect, validators

from webhattg.model import Announcement, Person, ResourceCount
from webhattg import utils

class AnnouncementController(identity.SecureResource):

    @expose()
    def default(self, *args, **kwargs):
        parts = http.request.path.rsplit('/')
        if len(parts) == 3 and parts[-1].isdigit():
            return self.announce(parts[-1])

    @expose()
    def index(self, *args, **kwargs):
        announce_id = self._active_announce_id()
        if announce_id:
            raise redirect('/announce/%s' % announce_id)
        return self.locate()

    @expose(template="webhattg.templates.announce.locate")
    def locate(self):
        all_announcements = Announcement.select()

        return dict(active_section='announce',
                    announcements=all_announcements)

    @expose(template="webhattg.templates.announce.announce")
    @tg.validate(validators=dict(announcement_id=validators.Int()))
    def announce(self, announcement_id):
        announcement = Announcement.get(announcement_id)
        hit_count = 0
        if announcement.category == 'Release':
            resource_name = 'se4pp-' + announcement.name.split('-')[-1] + '-setup.exe'
            resource = ResourceCount.by_resource_name(resource_name)
            hit_count = resource.hit_count

        if identity.current.user:
            user_person = Person.get(identity.current.user.id)
            groupids = utils.getGroups()
        else:
            user_person = 'visitor'
            groupids = []
        
        http.response.simpleCookie['active_announce_id'] = announcement_id
        http.response.simpleCookie['active_announce_id']['path'] = '/'

        return dict(active_section='announce',
                    announcement=announcement,
                    hit_count = hit_count,
                    user_person=user_person,
                    groupids=groupids)

    @expose(allow_json=True)
    def update_announcement(self, description):
        announcement = Announcement.get(self._active_announce_id())
        announcement.set(description=description)
        print description
        desc = textile(description.encode('utf-8'), encoding='utf-8', output='ascii')
        return dict(success=True, description=desc)
    
    def _active_announce_id(self):
        if http.request.simpleCookie.has_key('active_announce_id'):
            return int(http.request.simpleCookie['active_announce_id'].value)
        return None
