import turbogears as tg
from turbogears import expose, identity, redirect, validators
from webhattg.model import Category, Person, Ticket, Announcement
from sqlobject.sqlbuilder import AND

from tinymce import TinyMCE

ticketmce = TinyMCE(name="body")

class DashboardController(identity.SecureResource):

    @expose(template="webhattg.templates.dashboard.index")
    def index(self):
        open_tickets = Ticket.select(Ticket.q.status != 'closed')
        categories = Category.select()
        available_releases = Announcement.select(AND(Announcement.q.category == 'Release', Announcement.q.published == True)).reversed()
        available_release_count = Announcement.select(AND(Announcement.q.category == 'Release', Announcement.q.published == True)).count()
        #print available_release_count

        if identity.current.user:
            user_person = Person.get(identity.current.user.id)
        else:
            user_person = None
        
        return dict(active_section = 'dashboard',
                    tickets = open_tickets,
                    categories = categories,
                    user_person = user_person,
                    available_releases = available_releases,
                    available_release_count = available_release_count,
                    tinymce = ticketmce)

    @expose()
    @tg.validate(validators=dict(name=validators.UnicodeString(),reporter=validators.UnicodeString(),category_id=validators.Int(),body=validators.UnicodeString()))
    def new_ticket(self, name, reporter, category_id, body):
        if len(name) > 0 and len(reporter) > 0 and  len(body) > 0:
                category = Category.get(category_id)
                ticket = Ticket(name=name,
                                reporter=reporter,
                                category=category,
                                status='new',
                                priority='normal',
                                body=body,
                                owner=None,
                                resolution=1)

        raise redirect('/dashboard')
