# subcontroller/ticket.py

import turbogears as tg
import cherrypy as http
from turbogears import expose, identity, redirect, validators

from webhattg.model import Comment, Person, Ticket, Resolution
from webhattg import widgets, utils

from tinymce import TinyMCE
commentmce = TinyMCE(name="new_comment_body")

class TicketController(identity.SecureResource):

    @expose()
    def default(self, *args, **kwargs):
        parts = http.request.path.rsplit('/')
        if len(parts) == 3 and parts[-1].isdigit():
            return self.ticket(parts[-1])

    @expose()
    def index(self, *args, **kwargs):
        ticket_id = self._active_ticket_id()
        if ticket_id:
            raise redirect('/ticket/%s' % ticket_id)
        return self.locate()

    @expose(template="webhattg.templates.ticket.locate")
    def locate(self):
        all_tickets = Ticket.select()

        return dict(active_section='ticket',
                    tickets=all_tickets)

    @expose(template="webhattg.templates.ticket.ticket")
    @tg.validate(validators=dict(ticket_id=validators.Int()))
    def ticket(self, ticket_id):
        ticket = Ticket.get(ticket_id)

        if identity.current.user:
            user_person = Person.get(identity.current.user.id)
            groupids = utils.getGroups()
        else:
            user_person = 'visitor'
            groupids = []
        
        resolutions = Resolution.select()
        
        http.response.simpleCookie['active_ticket_id'] = ticket_id
        http.response.simpleCookie['active_ticket_id']['path'] = '/'

        people = Person.select()

        return dict(active_section='ticket',
                    ticket=ticket,
                    people=people,
                    user_person=user_person,
                    groupids=groupids,
                    resolutions=resolutions,
                    comments_widget=widgets.comments_widget,
                    comment_widget=widgets.comment_widget,
                    commentmce=commentmce)

    @expose(allow_json=True)
    @tg.validate(validators=dict(owner_id=validators.Int(), resolution_id=validators.Int()))
    def update_ticket(self, owner_id, resolution_id):
        owner = Person.get(owner_id)
        owner_content = owner.user_name
        
        if resolution_id != 1:
            # resolution_id != 1 means this ticket have a resolution
            closed = True
            status = 'closed'
        else:
            closed = False
            status = 'assigned'
        
        resolution = Resolution.get(resolution_id)
        ticket = Ticket.get(self._active_ticket_id())
        ticket.set(owner=owner, resolution=resolution, closed=closed, status=status)
        
        return dict(success=True, owner_content=owner_content)
    
    @expose()
    def create_comment(self, title, author, new_comment_body):
        print "###", new_comment_body
        if len(title) > 0 and len(author) > 0 and len(new_comment_body) > 0:
            ticket = Ticket.get(self._active_ticket_id())
            new_comment = Comment(author = author,
                                  title = title,
                                  body = new_comment_body,
                                  ticket = ticket)

        raise redirect('/ticket')

    def _active_ticket_id(self):
        if http.request.simpleCookie.has_key('active_ticket_id'):
            return int(http.request.simpleCookie['active_ticket_id'].value)
        return None
