import cherrypy
from turbogears import visit
from model import VisitIP, ResourceCount
from sqlobject import SQLObjectNotFound
from datetime import datetime

def ip_tracking_is_on():
    "Returns True if ip tracking is properly enabled, False otherwise."
    return cherrypy.config.get('visit.on', False) and cherrypy.config.get('visit.ip_tracking.on', False)

#Interface for the TurboGears extension
def start_extension():
    if not ip_tracking_is_on():
        return
    cherrypy.log( "Visit ip tracker starting" )

    #Register the plugin for the Visit Tracking framework
    visit.enable_visit_plugin( IPVisitPlugin() )
    
def shutdown_extension():
    if not ip_tracking_is_on():
        return
    cherrypy.log( "Visit ip tracker shutting down" )
    
class IPVisitPlugin(object):
    
    def __init__(self):
        cherrypy.log("IPVisitPlugin extension starting")
    
    def record_request(self, theVisit):
        # This method gets called on every single visit, so if you 
        # want to record something every time they make a request, this
        # is the place to do it.
        if theVisit.is_new:
            newVisitor = visit.sovisit.TG_Visit.by_visit_key(theVisit.key)
            print '>>>', newVisitor
            print '>>>', newVisitor.id
            self.new_visit(newVisitor.id)

        release_path = '/static/releases'
        if cherrypy.request.object_path.startswith(release_path):
            print '###', cherrypy.request.object_path.split('/')[-1]
            resource_name = cherrypy.request.object_path.split('/')[-1]
            try:
                resource = ResourceCount.by_resource_name(resource_name)
                print '###', resource
                resource.set(hit_count = resource.hit_count + 1,
                             last_access = datetime.now())
            except SQLObjectNotFound:
                ResourceCount(resource_name=resource_name, hit_count=1,
                              last_access = datetime.now())
        
    def new_visit(self, visit_id):
        # This method gets called the first time the visit is started.
        # I think IP tracking makes sense in here.
        v = visit.sovisit.TG_Visit.get(visit_id)
        
        # add a new visit ip object to the database
        VisitIP(visit=v, ip_address=cherrypy.request.remoteAddr)
